@@ -1,8 +1,14 @@
 {-# LANGUAGE CPP #-}
-{-# LANGUAGE MultiParamTypeClasses, FunctionalDependencies, FlexibleInstances, UndecidableInstances #-}
+{-# LANGUAGE FlexibleInstances #-}
+{-# LANGUAGE FunctionalDependencies #-}
+{-# LANGUAGE MultiParamTypeClasses #-}
+{-# LANGUAGE UndecidableInstances #-}
 #if __GLASGOW_HASKELL__ >= 702
 {-# LANGUAGE Safe #-}
 #endif
+#if __GLASGOW_HASKELL__ >= 706
+{-# LANGUAGE DeriveGeneric #-}
+#endif
 #if __GLASGOW_HASKELL__ >= 710
 {-# LANGUAGE AutoDeriveTypeable #-}
 #endif
@@ -69,6 +75,9 @@ module Data.FingerTree (
     ) where
 
 import Prelude hiding (null, reverse)
+#if MIN_VERSION_base(4,6,0)
+import GHC.Generics
+#endif
 #if MIN_VERSION_base(4,8,0)
 import qualified Prelude (null)
 #else
@@ -76,7 +85,7 @@ import Control.Applicative (Applicative(pure, (<*>)), (<$>))
 import Data.Monoid
 import Data.Foldable (Foldable(foldMap))
 #endif
-#if MIN_VERSION_base(4,9,0)
+#if (MIN_VERSION_base(4,9,0)) && !(MIN_VERSION_base(4,11,0))
 import Data.Semigroup
 #endif
 import Data.Foldable (toList)
@@ -89,14 +98,22 @@ infixl 5 |>, :>
 data ViewL s a
     = EmptyL        -- ^ empty sequence
     | a :< s a      -- ^ leftmost element and the rest of the sequence
-    deriving (Eq, Ord, Show, Read)
+    deriving (Eq, Ord, Show, Read
+#if __GLASGOW_HASKELL__ >= 706
+        , Generic
+#endif
+        )
 
 -- | View of the right end of a sequence.
 data ViewR s a
     = EmptyR        -- ^ empty sequence
     | s a :> a      -- ^ the sequence minus the rightmost element,
                     -- and the rightmost element
-    deriving (Eq, Ord, Show, Read)
+    deriving (Eq, Ord, Show, Read
+#if __GLASGOW_HASKELL__ >= 706
+        , Generic
+#endif
+        )
 
 instance (Functor s) => Functor (ViewL s) where
     fmap _ EmptyL    = EmptyL
@@ -125,7 +142,11 @@ data Digit a
     | Two a a
     | Three a a a
     | Four a a a a
-    deriving Show
+    deriving (Show
+#if __GLASGOW_HASKELL__ >= 706
+        , Generic
+#endif
+        )
 
 instance Foldable Digit where
     foldMap f (One a) = f a
@@ -149,7 +170,11 @@ instance (Measured v a) => Measured v (Digit a) where
 ---------------------------
 
 data Node v a = Node2 !v a a | Node3 !v a a a
-    deriving Show
+    deriving (Show
+#if __GLASGOW_HASKELL__ >= 706
+        , Generic
+#endif
+        )
 
 instance Foldable (Node v) where
     foldMap f (Node2 _ a b) = f a `mappend` f b
@@ -185,7 +210,13 @@ data FingerTree v a
     | Single a
     | Deep !v !(Digit a) (FingerTree v (Node v a)) !(Digit a)
 #if TESTING
-    deriving Show
+    deriving (Show
+#if __GLASGOW_HASKELL__ >= 706
+        , Generic
+#endif
+        )
+#elif __GLASGOW_HASKELL__ >= 706
+    deriving (Generic)
 #endif
 
 deep ::  (Measured v a) =>
@@ -293,6 +324,8 @@ mapWPDigit f v (Four a b c d) = Four (f v a) (f va b) (f vab c) (f vabc d)
 -- | Map all elements of the tree with a function that also takes the
 -- measure of the prefix to the left and of the suffix to the right of
 -- the element.
+--
+-- @since 0.1.2.0
 fmapWithContext :: (Measured v1 a1, Measured v2 a2) =>
     (v1 -> a1 -> v1 -> a2) -> FingerTree v1 a1 -> FingerTree v2 a2
 fmapWithContext f t = mapWCTree f mempty t mempty
@@ -427,6 +460,8 @@ traverseWPDigit f v (Four a b c d) = Four <$> f v a <*> f va b <*> f vab c <*> f
 -- | Traverse the tree from left to right with a function that also
 -- takes the measure of the prefix to the left and the measure of the
 -- suffix to the right of the element.
+--
+-- @since 0.1.2.0
 traverseWithContext :: (Measured v1 a1, Measured v2 a2, Applicative f) =>
     (v1 -> a1 -> v1 -> f a2) -> FingerTree v1 a1 -> f (FingerTree v2 a2)
 traverseWithContext f t = traverseWCTree f mempty t mempty
@@ -847,6 +882,8 @@ addDigits4 m1 (Four a b c d) e f g h (Four i j k l) m2 =
 
 -- | A result of 'search', attempting to find a point where a predicate
 -- on splits of the sequence changes from 'False' to 'True'.
+--
+-- @since 0.1.2.0
 data SearchResult v a
     = Position (FingerTree v a) a (FingerTree v a)
         -- ^ A tree opened at a particular element: the prefix to the
@@ -861,7 +898,11 @@ data SearchResult v a
         -- ^ No position in the tree, returned if the predicate is 'True'
         -- at the left end and 'False' at the right end.  This will not
         -- occur if the predicate in monotonic on the tree.
-    deriving (Eq, Ord, Show)
+    deriving (Eq, Ord, Show
+#if __GLASGOW_HASKELL__ >= 706
+        , Generic
+#endif
+        )
 
 -- | /O(log(min(i,n-i)))/. Search a sequence for a point where a predicate
 -- on splits of the sequence changes from 'False' to 'True'.
@@ -891,6 +932,8 @@ data SearchResult v a
 --
 -- For predictable results, one should ensure that there is only one such
 -- point, i.e. that the predicate is /monotonic/ on @t@.
+--
+-- @since 0.1.2.0
 search :: (Measured v a) =>
     (v -> v -> Bool) -> FingerTree v a -> SearchResult v a
 search p t
